// • ▌ ▄ ·.  ▄▄▄·  ▄▄ • ▪   ▄▄· ▄▄▄▄·  ▄▄▄·  ▐▄▄▄  ▄▄▄ .
// ·██ ▐███▪▐█ ▀█ ▐█ ▀ ▪██ ▐█ ▌▪▐█ ▀█▪▐█ ▀█ •█▌ ▐█▐▌·
// ▐█ ▌▐▌▐█·▄█▀▀█ ▄█ ▀█▄▐█·██ ▄▄▐█▀▀█▄▄█▀▀█ ▐█▐ ▐▌▐▀▀▀
// ██ ██▌▐█▌▐█ ▪▐▌▐█▄▪▐█▐█▌▐███▌██▄▪▐█▐█ ▪▐▌██▐ █▌▐█▄▄▌
// ▀▀  █▪▀▀▀ ▀  ▀ ·▀▀▀▀ ▀▀▀·▀▀▀ ·▀▀▀▀  ▀  ▀ ▀▀  █▪ ▀▀▀
//      Magicbane Emulator Project © 2013 - 2022
//                www.magicbane.com


package engine.mobileAI.utilities;

import engine.Enum.*;
import engine.gameManager.ChatManager;
import engine.gameManager.CombatManager;
import engine.math.Vector3fImmutable;
import engine.net.DispatchMessage;
import engine.net.client.msg.TargetedActionMsg;
import engine.objects.*;
import engine.server.MBServerStatics;
import org.pmw.tinylog.Logger;

import java.util.concurrent.ThreadLocalRandom;

import static engine.math.FastMath.sqr;
import static java.lang.Math.pow;

public class CombatUtilities {

    public static boolean inRangeToAttack(Mob agent, AbstractWorldObject target) {

        if (Float.isNaN(agent.getLoc().x))
            return false;

        try {
            Vector3fImmutable sl = agent.getLoc();
            Vector3fImmutable tl = target.getLoc();

            //add Hitbox's to range.
            float range = agent.getRange();
            range += CombatManager.calcHitBox(target) + CombatManager.calcHitBox(agent);

            return !(sl.distanceSquared(tl) > sqr(range));
        } catch (Exception e) {
            Logger.error(e.toString());
            return false;
        }

    }

    public static boolean inRange2D(AbstractWorldObject entity1, AbstractWorldObject entity2, double range) {
        return entity1.getLoc().distanceSquared2D(entity2.getLoc()) < range * range;
    }

    public static void swingIsBlock(Mob agent, AbstractWorldObject target, int animation) {

        if (!target.isAlive())
            return;

        TargetedActionMsg msg = new TargetedActionMsg(agent, animation, target, MBServerStatics.COMBAT_SEND_BLOCK);

        if (target.getObjectType() == GameObjectType.PlayerCharacter)
            DispatchMessage.dispatchMsgToInterestArea(target, msg, DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE, true, false);
        else
            DispatchMessage.sendToAllInRange(agent, msg);

    }

    public static void swingIsParry(Mob agent, AbstractWorldObject target, int animation) {

        if (!target.isAlive())
            return;

        TargetedActionMsg msg = new TargetedActionMsg(agent, animation, target, MBServerStatics.COMBAT_SEND_PARRY);

        if (target.getObjectType() == GameObjectType.PlayerCharacter)
            DispatchMessage.dispatchMsgToInterestArea(target, msg, DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE, true, false);
        else
            DispatchMessage.sendToAllInRange(agent, msg);

    }

    public static void swingIsDodge(Mob agent, AbstractWorldObject target, int animation) {

        if (!target.isAlive())
            return;

        TargetedActionMsg msg = new TargetedActionMsg(agent, animation, target, MBServerStatics.COMBAT_SEND_DODGE);

        if (target.getObjectType() == GameObjectType.PlayerCharacter)
            DispatchMessage.dispatchMsgToInterestArea(target, msg, DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE, true, false);
        else
            DispatchMessage.sendToAllInRange(agent, msg);
    }

    public static void swingIsDamage(Mob agent, AbstractWorldObject target, float damage, int animation) {
        if (agent.isSiege() == true) {
            damage = ThreadLocalRandom.current().nextInt(1000) + 1500;
        }
        float trueDamage = damage;

        if (!target.isAlive())
            return;

        if (AbstractWorldObject.IsAbstractCharacter(target))
            trueDamage = ((AbstractCharacter) target).modifyHealth(-damage, agent, false);
        else if (target.getObjectType() == GameObjectType.Building)
            trueDamage = ((Building) target).modifyHealth(-damage, agent);

        //Don't send 0 damage kay thanx.

        if (trueDamage == 0)
            return;

        TargetedActionMsg msg = new TargetedActionMsg(agent, target, damage, animation);

        if (target.getObjectType() == GameObjectType.PlayerCharacter)
            DispatchMessage.dispatchMsgToInterestArea(target, msg, DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE, true, false);
        else
            DispatchMessage.sendToAllInRange(agent, msg);

        //check damage shields
        if (AbstractWorldObject.IsAbstractCharacter(target) && target.isAlive() && target.getObjectType() != GameObjectType.Mob)
            CombatManager.handleDamageShields(agent, (AbstractCharacter) target, damage);
    }

    public static boolean canSwing(Mob agent) {
        return (agent.isAlive() && !agent.getBonuses().getBool(ModType.Stunned, SourceType.None));
    }

    public static void swingIsMiss(Mob agent, AbstractWorldObject target, int animation) {

        TargetedActionMsg msg = new TargetedActionMsg(agent, target, 0f, animation);

        if (target.getObjectType() == GameObjectType.PlayerCharacter)
            DispatchMessage.dispatchMsgToInterestArea(target, msg, DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE, true, false);
        else
            DispatchMessage.sendToAllInRange(agent, msg);

    }

    public static boolean triggerDefense(Mob agent, AbstractWorldObject target) {
        int defenseScore = 0;
        int attackScore = agent.getAtrHandOne();
        switch (target.getObjectType()) {
            case PlayerCharacter:
                defenseScore = ((AbstractCharacter) target).getDefenseRating();
                break;
            case Mob:

                Mob mob = (Mob) target;
                if (mob.isSiege())
                    defenseScore = attackScore;
                break;
            case Building:
                return false;
        }

        int hitChance;
        if (attackScore > defenseScore || defenseScore == 0)
            hitChance = 94;
        else if (attackScore == defenseScore && target.getObjectType() == GameObjectType.Mob)
            hitChance = 10;
        else {
            float dif = attackScore / defenseScore;
            if (dif <= 0.8f)
                hitChance = 4;
            else
                hitChance = ((int) (450 * (dif - 0.8f)) + 4);
            if (target.getObjectType() == GameObjectType.Building)
                hitChance = 100;
        }
        return ThreadLocalRandom.current().nextInt(100) > hitChance;
    }

    public static boolean triggerBlock(Mob agent, AbstractWorldObject ac) {
        return triggerPassive(agent, ac, "Block");
    }

    public static boolean triggerParry(Mob agent, AbstractWorldObject ac) {
        return triggerPassive(agent, ac, "Parry");
    }

    public static boolean triggerDodge(Mob agent, AbstractWorldObject ac) {
        return triggerPassive(agent, ac, "Dodge");
    }

    public static boolean triggerPassive(Mob agent, AbstractWorldObject ac, String type) {
        float chance = 0;
        if (AbstractWorldObject.IsAbstractCharacter(ac))
            chance = ((AbstractCharacter) ac).getPassiveChance(type, agent.getLevel(), true);

        if (chance > 75f)
            chance = 75f;
        if (agent.isSiege() && AbstractWorldObject.IsAbstractCharacter(ac))
            chance = 100;

        return ThreadLocalRandom.current().nextInt(100) < chance;
    }

    public static void combatCycle(Mob agent, AbstractWorldObject target, boolean mainHand, ItemBase wb) {

        if (!agent.isAlive() || !target.isAlive())
            return;

        if (target.getObjectType() == GameObjectType.PlayerCharacter)
            if (!((PlayerCharacter) target).isActive())
                return;

        int anim = 75;
        float speed;

        if (mainHand)
            speed = agent.getSpeedHandOne();
        else
            speed = agent.getSpeedHandTwo();

        DamageType dt = DamageType.Crush;

        if (agent.isSiege())
            dt = DamageType.Siege;
        if (wb != null) {
            anim = CombatManager.getSwingAnimation(wb, null, mainHand);
            dt = wb.getDamageType();
        } else if (!mainHand)
            return;

        Resists res = null;
        PlayerBonuses bonus = null;

        switch (target.getObjectType()) {
            case Building:
                res = ((Building) target).getResists();
                break;
            case PlayerCharacter:
                res = ((PlayerCharacter) target).getResists();
                bonus = ((PlayerCharacter) target).getBonuses();
                break;
            case Mob:
                Mob mob = (Mob) target;
                res = mob.getResists();
                bonus = ((Mob) target).getBonuses();
                break;
        }

        //must not be immune to all or immune to attack

        if (bonus != null && !bonus.getBool(ModType.NoMod, SourceType.ImmuneToAttack))
            if (res != null && (res.immuneToAll() || res.immuneToAttacks() || res.immuneTo(dt)))
                return;

        int passiveAnim = CombatManager.getSwingAnimation(wb, null, mainHand);
        if (canSwing(agent)) {
            if (triggerDefense(agent, target)) {
                swingIsMiss(agent, target, passiveAnim);
                return;
            } else if (triggerDodge(agent, target)) {
                swingIsDodge(agent, target, passiveAnim);
                return;
            } else if (triggerParry(agent, target)) {
                swingIsParry(agent, target, passiveAnim);

                return;
            } else if (triggerBlock(agent, target)) {
                swingIsBlock(agent, target, passiveAnim);
                return;
            }

            swingIsDamage(agent, target, determineDamage(agent), anim);


            if (agent.getWeaponPower() != null)
                agent.getWeaponPower().attack(target, MBServerStatics.ONE_MINUTE);
        }

        if (target.getObjectType().equals(GameObjectType.PlayerCharacter)) {
            PlayerCharacter player = (PlayerCharacter) target;

            if (player.getDebug(64))
                ChatManager.chatSayInfo(player, "Debug Combat: Mob UUID " + agent.getObjectUUID() + " || Building ID  = " + agent.getBuildingID() + " || Floor = " + agent.getInFloorID() + " || Level = " + agent.getInBuilding());//combat debug
        }

        //SIEGE MONSTERS DO NOT ATTACK GUARDSs
        if (target.getObjectType() == GameObjectType.Mob)
            if (((Mob) target).isSiege())
                return;

        //handle the retaliate

        if (AbstractWorldObject.IsAbstractCharacter(target))
            CombatManager.handleRetaliate((AbstractCharacter) target, agent);

        if (target.getObjectType() == GameObjectType.Mob) {
            Mob targetMob = (Mob) target;
            if (targetMob.isSiege())
                return;
        }

    }

    public static float determineDamage(Mob agent) {

        //early exit for null

        if (agent == null)
            return 0;

        AbstractWorldObject target = agent.getCombatTarget();

        if (target == null)
            return 0;

        int damage = 0;

        DamageType dt = getDamageType(agent);
        damage = ThreadLocalRandom.current().nextInt((int)getMinDmg(agent), (int)getMaxDmg(agent) + 1);
        if (AbstractWorldObject.IsAbstractCharacter(target)) {
            if (((AbstractCharacter) target).isSit()) {
                damage *= 2.5f; //increase damage if sitting
            }
            return (int) (((AbstractCharacter) target).getResists().getResistedDamage(agent, (AbstractCharacter) target, dt, damage, 0));
        }
        if (target.getObjectType() == GameObjectType.Building) {
            Building building = (Building) target;
            Resists resists = building.getResists();
            return (int) ((damage * (1 - (resists.getResist(dt, 0) / 100))));
        }
        return damage;
    }

    public static DamageType getDamageType(Mob agent) {
        DamageType dt = DamageType.Crush;
        if (agent.getEquip().get(1) != null) {
            return agent.getEquip().get(1).getItemBase().getDamageType();
        }
        if (agent.getEquip().get(2) != null && agent.getEquip().get(2).getItemBase().isShield() == false) {
            return agent.getEquip().get(2).getItemBase().getDamageType();
        }
        return dt;
    }

    public static double getMinDmg(Mob agent) {
        if(agent.equip.get(2) != null && !agent.equip.get(2).getItemBase().isShield())
            return agent.minDamageHandTwo;
        else return agent.minDamageHandOne;
    }

    public static double getMaxDmg(Mob agent) {
        if(agent.equip.get(2) != null && !agent.equip.get(2).getItemBase().isShield())
            return agent.maxDamageHandTwo;
        else return agent.maxDamageHandOne;
    }

}
